@@ -8,12 +8,12 @@
 //! one column. Each cell in the table containsa binary vector of flags for each fine channel in
 //! the coarse channel.
 
-use crate::cxx_aoflagger::ffi::CxxFlagMask;
-use crate::error::BirliError;
-use crate::io::error::{
+use super::error::{
     IOError,
     IOError::{FitsIO, FitsOpen, InvalidFlagFilenameTemplate, InvalidGpuBox, MwafInconsistent},
 };
+use crate::cxx_aoflagger::ffi::CxxFlagMask;
+use crate::error::BirliError;
 use clap::crate_version;
 use cxx::UniquePtr;
 use fitsio::tables::{ColumnDataDescription, ColumnDataType, ConcreteColumnDescription};
@@ -260,6 +260,139 @@ impl FlagFileSet {
         Ok(header)
     }
 
+    /// Write flags to disk, given an observation's [`mwalib::CorrelatorContext`], and a
+    /// vector of [`CxxFlagMask`]s for each baseline in the observation.
+    ///
+    /// The filename template should contain two or 3 percentage (`%`) characters which will be replaced
+    /// by the gpubox id or channel number (depending on correlator type). See [`FlagFileSet::new`]
+    ///
+    /// # Errors
+    ///
+    /// Will error if the gpubox ids this flagset was initialized with is not contained in the
+    /// provided [`mwalib::CorrelatorContext`].
+    ///
+    pub fn write_baseline_flagmasks(
+        &mut self,
+        context: &CorrelatorContext,
+        baseline_flagmasks: &[UniquePtr<CxxFlagMask>],
+        img_coarse_chan_idxs: &[usize],
+    ) -> Result<(), IOError> {
+        // let gpubox_chan_numbers: BTreeMap<usize, usize> = context
+        //     .coarse_chans
+        //     .iter()
+        //     .map(|chan| (chan.gpubox_number, chan.corr_chan_number))
+        //     .collect();
+        let gpubox_to_chan_idx: BTreeMap<usize, usize> = context
+            .coarse_chans
+            .iter()
+            .enumerate()
+            .map(|(chan_idx, chan)| (chan.gpubox_number, chan_idx))
+            .collect();
+
+        for (&gpubox_id, fptr) in self.gpubox_fptrs.iter_mut() {
+            let primary_hdu = fits_open_hdu!(fptr, 0)?;
+            let header = FlagFileHeaders::from_gpubox_context(gpubox_id, context);
+            let num_baselines = header.num_ants * (header.num_ants + 1) / 2;
+            let num_fine_chans_per_coarse = header.num_channels;
+            FlagFileSet::write_primary_hdu(fptr, &primary_hdu, &header)?;
+            // let chan_number = match gpubox_chan_numbers.get(&gpubox_id) {
+            //     Some(chan_number) => chan_number,
+            //     None => {
+            //         return Err(InvalidGpuBox {
+            //             expected: format!("{:?}", gpubox_chan_numbers.keys()),
+            //             found: format!("{}", gpubox_id),
+            //         })
+            //     }
+            // };
+            let coarse_chan_idx = match gpubox_to_chan_idx.get(&gpubox_id) {
+                Some(chan_number) => chan_number,
+                None => {
+                    return Err(InvalidGpuBox {
+                        expected: format!("{:?}", gpubox_to_chan_idx.keys()),
+                        found: format!("{}", gpubox_id),
+                    })
+                }
+            };
+            let img_coarse_chan_idx = match img_coarse_chan_idxs
+                .iter()
+                .position(|idx| idx == coarse_chan_idx)
+            {
+                Some(img_coarse_chan_idx) => img_coarse_chan_idx,
+                None => {
+                    return Err(InvalidGpuBox {
+                        expected: format!("{:?}", img_coarse_chan_idxs),
+                        found: format!("{}", coarse_chan_idx),
+                    })
+                }
+            };
+            let flags_colname = "FLAGS";
+            let table_hdu = fptr.create_table(
+                "EXTNAME".to_string(),
+                &[ConcreteColumnDescription {
+                    name: flags_colname.to_string(),
+                    data_type: ColumnDataDescription::vector(
+                        ColumnDataType::Bit,
+                        num_fine_chans_per_coarse,
+                    ),
+                }],
+            )?;
+            FlagFileSet::write_table_hdu(fptr, &table_hdu, &header)?;
+
+            let mut status = 0;
+            for (baseline_idx, flagmask) in baseline_flagmasks.iter().enumerate() {
+                assert!(baseline_idx < num_baselines);
+                // Flag buffer can be thought of as a 2D buffer of bools,
+                // indexed by channel (y), then timestep (x). flag stride is the width
+                let flag_buffer = flagmask.Buffer();
+                let flag_stride = flagmask.HorizontalStride();
+                let num_timesteps = flagmask.Width();
+                let num_channels = flagmask.Height();
+                assert_eq!(
+                    num_channels,
+                    img_coarse_chan_idxs.len() * num_fine_chans_per_coarse
+                );
+                for img_timestep_idx in 0..num_timesteps {
+                    let row_idx = (img_timestep_idx * num_baselines) + baseline_idx;
+                    // All flags for img_timestep_idx
+                    let timestep_flags = flag_buffer
+                        .iter()
+                        .skip(img_timestep_idx)
+                        .step_by(flag_stride);
+
+                    // All flags for timestep_idx and img_coarse_chan_idx
+                    let mut cell: Vec<i8> = timestep_flags
+                        .skip(img_coarse_chan_idx * num_fine_chans_per_coarse)
+                        .take(num_fine_chans_per_coarse)
+                        .map(|&flag| i8::from(flag))
+                        .collect();
+                    unsafe {
+                        fitsio_sys::ffpclx(
+                            fptr.as_raw(),
+                            1,
+                            1 + row_idx as i64,
+                            1,
+                            cell.len() as i64,
+                            cell.as_mut_ptr(),
+                            &mut status,
+                        );
+                    }
+                    fitsio::errors::check_status(status).map_err(|e| FitsIO {
+                        fits_error: e,
+                        fits_filename: String::from(&fptr.filename),
+                        hdu_num: 1,
+                        source_file: file!(),
+                        source_line: line!(),
+                    })?;
+                }
+            }
+        }
+        Ok(())
+    }
+}
+
+/// TODO: These are just for tests, and should be deprecated.
+/// TODO: Why doesn't #[cfg(test)] work?
+impl FlagFileSet {
     // pub fn read_validated_header(
     //     context: &CorrelatorContext,
     //     fptr: &mut FitsFile,
@@ -298,97 +431,6 @@ impl FlagFileSet {
     //     Ok(headers)
     // }
 
-    /// Write flags to disk, given an observation's [`mwalib::CorrelatorContext`], and a
-    /// vector of [`CxxFlagMask`]s for each baseline in the observation.
-    ///
-    /// The filename template should contain two or 3 percentage (`%`) characters which will be replaced
-    /// by the gpubox id or channel number (depending on correlator type). See [`FlagFileSet::new`]
-    ///
-    /// # Errors
-    ///
-    /// Will error if the gpubox ids this flagset was initialized with is not contained in the
-    /// provided [`mwalib::CorrelatorContext`].
-    ///
-    pub fn write_baseline_flagmasks(
-        &mut self,
-        context: &CorrelatorContext,
-        baseline_flagmasks: Vec<UniquePtr<CxxFlagMask>>,
-    ) -> Result<(), IOError> {
-        let gpubox_chan_numbers: BTreeMap<usize, usize> = context
-            .coarse_chans
-            .iter()
-            .map(|chan| (chan.gpubox_number, chan.corr_chan_number))
-            .collect();
-
-        for (&gpubox_id, fptr) in self.gpubox_fptrs.iter_mut() {
-            let primary_hdu = fits_open_hdu!(fptr, 0)?;
-            let header = FlagFileHeaders::from_gpubox_context(gpubox_id, context);
-            let num_baselines = header.num_ants * (header.num_ants + 1) / 2;
-            let num_fine_chans_per_coarse = header.num_channels;
-            FlagFileSet::write_primary_hdu(fptr, &primary_hdu, &header)?;
-            let chan_number = match gpubox_chan_numbers.get(&gpubox_id) {
-                Some(chan_number) => chan_number,
-                None => {
-                    return Err(InvalidGpuBox {
-                        expected: format!("{:?}", gpubox_chan_numbers.keys()),
-                        found: format!("{}", gpubox_id),
-                    })
-                }
-            };
-            let flags_colname = "FLAGS";
-            let table_hdu = fptr.create_table(
-                "EXTNAME".to_string(),
-                &[ConcreteColumnDescription {
-                    name: flags_colname.to_string(),
-                    data_type: ColumnDataDescription::vector(
-                        ColumnDataType::Bit,
-                        num_fine_chans_per_coarse,
-                    ),
-                }],
-            )?;
-            FlagFileSet::write_table_hdu(fptr, &table_hdu, &header)?;
-
-            let mut status = 0;
-            for (baseline_idx, flagmask) in baseline_flagmasks.iter().enumerate() {
-                // TODO: Assert baseline_idx < num_baselines?
-                let flag_buffer = flagmask.Buffer();
-                let flag_stride = flagmask.HorizontalStride();
-                let num_timesteps = flagmask.Width();
-                for timestep_idx in 0..num_timesteps {
-                    let row_idx = (timestep_idx * num_baselines) + baseline_idx;
-                    // TODO: document this, it's not super clear
-                    let mut cell: Vec<i8> = flag_buffer
-                        .iter()
-                        .skip(timestep_idx)
-                        .step_by(flag_stride)
-                        .skip(chan_number * num_fine_chans_per_coarse)
-                        .take(num_fine_chans_per_coarse)
-                        .map(|&flag| i8::from(flag))
-                        .collect();
-                    unsafe {
-                        fitsio_sys::ffpclx(
-                            fptr.as_raw(),
-                            1,
-                            1 + row_idx as i64,
-                            1,
-                            cell.len() as i64,
-                            cell.as_mut_ptr(),
-                            &mut status,
-                        );
-                    }
-                    fitsio::errors::check_status(status).map_err(|e| FitsIO {
-                        fits_error: e,
-                        fits_filename: String::from(&fptr.filename),
-                        hdu_num: 1,
-                        source_file: file!(),
-                        source_line: line!(),
-                    })?;
-                }
-            }
-        }
-        Ok(())
-    }
-
     fn read_flags_raw(
         fptr: &mut FitsFile,
         flags_raw: &mut [i8],
@@ -794,9 +836,11 @@ mod tests {
         let mut idx = 0;
         let num_fine_chans_per_coarse = context.metafits_context.num_corr_fine_chans_per_coarse;
 
+        let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
+        let num_img_coarse_chans = img_coarse_chan_idxs.len();
+
         let width = context.num_common_timesteps;
-        let height = context.num_common_coarse_chans
-            * context.metafits_context.num_corr_fine_chans_per_coarse;
+        let height = num_img_coarse_chans * context.metafits_context.num_corr_fine_chans_per_coarse;
 
         let aoflagger = unsafe { cxx_aoflagger_new() };
         let mut baseline_flagmasks: Vec<UniquePtr<CxxFlagMask>> = context
@@ -806,15 +850,15 @@ mod tests {
             .map(|_| unsafe { aoflagger.MakeFlagMask(width, height, false) })
             .collect();
 
-        for (coarse_chan_idx, _) in context.coarse_chans.iter().enumerate() {
-            for (timestep_idx, _) in context.timesteps.iter().enumerate() {
+        for img_coarse_chan_idx in 0..num_img_coarse_chans {
+            for img_timestep_idx in 0..width {
                 for flag_mask_ptr in baseline_flagmasks.iter_mut() {
                     let flag_stride = flag_mask_ptr.HorizontalStride();
                     let flag_buf = flag_mask_ptr.pin_mut().BufferMut();
                     for fine_chan_idx in 0..num_fine_chans_per_coarse {
                         let flag_offset_y =
-                            coarse_chan_idx * num_fine_chans_per_coarse + fine_chan_idx;
-                        let flag_idx = flag_offset_y * flag_stride + timestep_idx;
+                            img_coarse_chan_idx * num_fine_chans_per_coarse + fine_chan_idx;
+                        let flag_idx = flag_offset_y * flag_stride + img_timestep_idx;
                         assert!(flag_idx < flag_stride * height);
                         dbg!(flag_idx, fine_chan_idx, idx, 1 << fine_chan_idx & idx);
                         flag_buf[flag_idx] = 1 << fine_chan_idx & idx != 0;
@@ -824,26 +868,16 @@ mod tests {
             }
         }
 
-        {
-            let mut flag_file_set = FlagFileSet::new(
-                filename_template.to_str().unwrap(),
-                &gpubox_ids,
-                context.mwa_version,
-            )
+        let mut flag_file_set = FlagFileSet::new(
+            filename_template.to_str().unwrap(),
+            &gpubox_ids,
+            context.mwa_version,
+        )
+        .unwrap();
+        flag_file_set
+            .write_baseline_flagmasks(&context, &baseline_flagmasks, &img_coarse_chan_idxs)
             .unwrap();
-            flag_file_set
-                .write_baseline_flagmasks(&context, baseline_flagmasks)
-                .unwrap();
-        }
-
-        for &gpubox_id in gpubox_ids.iter() {
-            let flag_path = tmp_dir
-                .path()
-                .join(format!("Flagfile{:03}.mwaf", gpubox_id));
-            let mut flag_fptr = FitsFile::open(flag_path).unwrap();
-            let table_hdu = flag_fptr.hdu(1).unwrap();
-            dbg!(table_hdu);
-        }
+        drop(flag_file_set);
 
         let mut flag_file_set = FlagFileSet::open(
             filename_template.to_str().unwrap(),
