@@ -1,12 +1,12 @@
 use clap::{crate_authors, crate_description, crate_name, crate_version, App, Arg, SubCommand};
-use log::{debug, info};
+use log::{debug, info, trace};
 // use log::{log_enabled, Level};
-use std::{env, ffi::OsString, fmt::Debug};
+use std::{env, ffi::OsString, fmt::Debug, path::Path};
 
 use birli::{
     context_to_baseline_imgsets, correct_cable_lengths, cxx_aoflagger_new, flag_imgsets_existing,
     get_antenna_flags, get_aoflagger_version_string, get_flaggable_timesteps,
-    init_baseline_flagmasks, write_flags,
+    init_baseline_flagmasks, io::write_uvfits, write_flags,
 };
 // use birli::util::{dump_flagmask, dump_imgset};
 use mwalib::CorrelatorContext;
@@ -39,16 +39,20 @@ where
         .arg(
             Arg::with_name("flag-template")
                 .short("f")
-                .takes_value(true)
-                .required(true) // TODO: specify a default that works with mwa-ord and mwax
+                .takes_value(true)// TODO: specify a default that works with mwa-ord and mwax
                 .help("Sets the template used to name flag files. Percents are substituted for the zero-prefixed GPUBox ID, which can be up to 3 characters log. Similar to -o in Cotter. Example: FlagFile%%%.mwaf")
         )
+        .arg(
+            Arg::with_name("uvfits-out")
+                .short("u")
+                .takes_value(true)
+                .help("Filename for uvfits output. Similar to -o in Cotter. Example: 1196175296.uvfits")
+        )
         .arg(
             Arg::with_name("no-cable-delay")
                 .long("no-cable-delay")
                 .takes_value(false)
                 .required(false)
-                .number_of_values(0)
                 .help("Do not perform cable length corrections.")
         )
         // TODO: implement specify flag strategy
@@ -69,7 +73,8 @@ where
     if let Some(aoflagger_matches) = matches.subcommand_matches("aoflagger") {
         let aoflagger = unsafe { cxx_aoflagger_new() };
         let metafits_path = aoflagger_matches.value_of("metafits").unwrap();
-        let flag_template = aoflagger_matches.value_of("flag-template").unwrap();
+        let flag_template = aoflagger_matches.value_of("flag-template");
+        let uvfits_out = aoflagger_matches.value_of("uvfits-out");
         let fits_files: Vec<&str> = aoflagger_matches.values_of("fits-files").unwrap().collect();
         let context = match CorrelatorContext::new(&metafits_path, &fits_files) {
             Ok(context) => context,
@@ -77,17 +82,36 @@ where
         };
         debug!("mwalib correlator context:\n{}", &context);
         let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
+        trace!("img_coarse_chan_idxs: {:?}", img_coarse_chan_idxs);
         let img_timestep_idxs = match get_flaggable_timesteps(&context) {
             Ok(timestep_idxs) => timestep_idxs,
             Err(err) => panic!("unable to determine flaggable timesteps: {}", err),
         };
+        trace!("img_timestep_idxs: {:?}", img_timestep_idxs);
+
+        let antenna_flags = get_antenna_flags(&context);
+        // trace!("antenna_flags: {:?}", antenna_flags);
+        trace!(
+            "antenna_flags: {:?}",
+            antenna_flags
+                .iter()
+                .enumerate()
+                .filter_map(|(idx, &flag)| {
+                    if flag {
+                        Some(idx)
+                    } else {
+                        None
+                    }
+                })
+                .collect::<Vec<_>>()
+        );
 
         let mut baseline_flagmasks = init_baseline_flagmasks(
             &aoflagger,
             &context,
             &img_coarse_chan_idxs,
             &img_timestep_idxs,
-            Some(get_antenna_flags(&context)),
+            Some(antenna_flags),
         );
 
         let mut baseline_imgsets = context_to_baseline_imgsets(
@@ -102,11 +126,20 @@ where
         let no_cable_delays = aoflagger_matches.is_present("no-cable-delay");
         let cable_delays_applied = context.metafits_context.cable_delays_applied;
         if !cable_delays_applied && !no_cable_delays {
-            correct_cable_lengths(&context, &mut baseline_imgsets);
+            debug!(
+                "Applying cable delays. applied: {}, desired: {}",
+                cable_delays_applied, !no_cable_delays
+            );
+            correct_cable_lengths(&context, &mut baseline_imgsets, img_coarse_chan_idxs);
+        } else {
+            debug!(
+                "Skipping cable delays. applied: {}, desired: {}",
+                cable_delays_applied, !no_cable_delays
+            );
         }
 
         let strategy_filename = &aoflagger.FindStrategyFileMWA();
-
+        debug!("flagging with strategy {}", strategy_filename);
         flag_imgsets_existing(
             &aoflagger,
             &strategy_filename,
@@ -115,12 +148,29 @@ where
             true,
         );
 
-        let gpubox_ids: Vec<usize> = context
-            .common_coarse_chan_indices
-            .iter()
-            .map(|&chan| context.coarse_chans[chan].gpubox_number)
-            .collect();
-        write_flags(&context, baseline_flagmasks, flag_template, &gpubox_ids);
+        if let Some(flag_template) = flag_template {
+            write_flags(
+                &context,
+                &baseline_flagmasks,
+                flag_template,
+                &img_coarse_chan_idxs,
+            )
+            .unwrap();
+        }
+
+        if let Some(uvfits_out) = uvfits_out {
+            let baseline_idxs = (0..context.metafits_context.num_baselines).collect::<Vec<_>>();
+            write_uvfits(
+                Path::new(uvfits_out),
+                &context,
+                &baseline_idxs,
+                &baseline_imgsets,
+                &baseline_flagmasks,
+                &img_timestep_idxs,
+                &img_coarse_chan_idxs,
+            )
+            .unwrap();
+        }
     }
 }
 
@@ -234,7 +284,7 @@ mod tests {
     #[test]
     fn aoflagger_outputs_flags() {
         let tmp_dir = tempdir().unwrap();
-        let filename_template = tmp_dir.path().join("Flagfile%%.mwaf");
+        let mwaf_path_template = tmp_dir.path().join("Flagfile%%.mwaf");
 
         let metafits_path = "tests/data/1247842824_flags/1247842824.metafits";
         let gpufits_paths =
@@ -246,7 +296,7 @@ mod tests {
             "-m",
             metafits_path,
             "-f",
-            filename_template.to_str().unwrap(),
+            mwaf_path_template.to_str().unwrap(),
         ];
         args.extend_from_slice(&gpufits_paths);
         dbg!(&args);
@@ -264,7 +314,7 @@ mod tests {
         assert!(!gpubox_ids.is_empty());
 
         let mut birli_flag_file_set = FlagFileSet::open(
-            filename_template.to_str().unwrap(),
+            mwaf_path_template.to_str().unwrap(),
             &gpubox_ids,
             context.mwa_version,
         )
@@ -285,5 +335,31 @@ mod tests {
         );
     }
 
-    // TODO: test uvfits output with / without cable-delay
+    #[test]
+    fn aoflagger_outputs_uvfits() {
+        let tmp_dir = tempdir().unwrap();
+        let uvfits_path = tmp_dir.path().join("1247842824.uvfits");
+
+        let metafits_path = "tests/data/1247842824_flags/1247842824.metafits";
+        let gpufits_paths =
+            vec!["tests/data/1247842824_flags/1247842824_20190722150008_gpubox01_00.fits"];
+
+        let mut args = vec![
+            "birli",
+            "aoflagger",
+            "-m",
+            metafits_path,
+            "-u",
+            uvfits_path.to_str().unwrap(),
+            "--no-cable-delay",
+        ];
+        args.extend_from_slice(&gpufits_paths);
+        dbg!(&args);
+
+        main_with_args(&args);
+
+        assert!(uvfits_path.exists());
+
+        assert!(uvfits_path.metadata().unwrap().len() > 0);
+    }
 }
